__author__ = 'mahajrod'

from RouToolPa.Tools.BLAST.BLAST import *
from RouToolPa.Tools.BLAST.BLASTPlus import *


BLAST = BLAST()
BLASTn = BLASTn()
BLASTp = BLASTp()
BLASTPlus = BLASTPlus()
DustMasker = DustMasker()
BLASTDbCmd = BLASTDbCmd()
MakeBLASTDb = MakeBLASTDb()
Windowmasker = Windowmasker()



#!/usr/bin/env python

from RouToolPa.Tools.Abstract import JavaTool
from RouToolPa.Routines import VCFRoutines




class VariantFiltration(JavaTool):
    # http://www.broadinstitute.org/gatk/gatkdocs/org_broadinstitute_sting_gatk_walkers_filters_VariantFiltration.html
    # default filters for indel and snp filtration were taken from GATK BestPractice
    def __init__(self,  java_path="", max_threads=4, jar_path="", max_memory=None, timelog=None):
        JavaTool.__init__(self, "GenomeAnalysisTK.jar -T VariantFiltration", java_path=java_path,
                          max_threads=max_threads, jar_path=jar_path, max_memory=max_memory,
                          timelog=timelog)

    @staticmethod
    def parse_common_options(reference_file, input_vcf, output_vcf, filter_expression, filter_name):
        options = " -R %s" % reference_file
        options += " -V %s" % input_vcf
        options += " --filterExpression \'%s\'" % filter_expression
        options += " --filterName %s" % filter_name
        options += " -o %s" % output_vcf

        return options

    def filter(self, reference_file, input_vcf, output_vcf, filter_expression, filter_name, ):
        options = self.parse_common_options(reference_file, input_vcf, output_vcf, filter_expression, filter_name)

        self.execute(options=options)

    def filter_bad_SNP(self, reference_file, input_vcf, output_vcf, filter_name='ambiguous_snp', QD=2.0, FS=60.0,
                       MQ=40.0, MappingQualityRankSum=-12.5, ReadPosRankSum=-8.0):
        """
        "QD < 2.0 || FS > 60.0 || MQ < 40.0 || MQRankSum < -12.5 || ReadPosRankSum < -8.0"
        """
        filter_expression = "QD < %f || FS > %f || MQ < %f || MQRankSum < %f || ReadPosRankSum < %f" \
                           % (QD, FS, MQ, MappingQualityRankSum,  ReadPosRankSum)

        self.filter(reference_file, input_vcf, output_vcf, filter_expression, filter_name)

    def filter_bad_indel(self, reference_file, input_vcf, output_vcf, filter_name='ambiguous_indel', QD=2.0,
                         ReadPosRankSum=-20.0,  FS=200.0):
        """
        "QD < 2.0 || FS > 200.0 || ReadPosRankSum < -20.0"
        """
        filter_expression = "QD < %f || ReadPosRankSum < %f || FS > %f" % (QD, ReadPosRankSum, FS)
        self.filter(reference_file, input_vcf, output_vcf, filter_expression, filter_name)

    def filter_bad_variants(self, reference_file, input_vcf, output_prefix, snp_filter_name='ambiguous_snp', snp_QD=2.0,
                            snp_FS=60.0, snp_MQ=40.0, snp_HaplotypeScore=13.0, snp_MappingQualityRankSum=-12.5,
                            snp_ReadPosRankSum=-8.0, indel_filter_name='ambiguous_indel', indel_QD=2.0,
                            indel_ReadPosRankSum=-20.0, indel_FS=200.0, combine_vcf=False, sequence_dict_file=None,
                            picard_memory="1g", picard_dir=None):

        from RouToolPa.Tools.GATK import SelectVariants
        from RouToolPa.Tools.Picard import SortVcf
        snp_raw_vcf = "%s.snp.raw.vcf" % output_prefix
        indel_raw_vcf = "%s.indel.raw.vcf" % output_prefix

        snp_filtered_vcf = "%s.snp.with_filters.vcf" % output_prefix
        indel_filtered_vcf = "%s.indel.with_filters.vcf" % output_prefix

        snp_good_vcf = "%s.snp.good.vcf" % output_prefix
        indel_good_vcf = "%s.indel.good.vcf" % output_prefix

        unsorted_combined_filtered_vcf = "%s.combined.with_filters.unsorted.vcf" % output_prefix
        unsorted_combined_good_vcf = "%s.combined.good.unsorted.vcf" % output_prefix

        combined_filtered_vcf = "%s.combined.with_filters.sorted.vcf" % output_prefix
        combined_good_vcf = "%s.combined.good.sorted.vcf" % output_prefix


        SelectVariants.path = self.path
        SortVcf.jar_path = picard_dir
        SortVcf.max_memory = picard_memory
        #CombineVariants.jar_path = self.jar_path

        SelectVariants.get_SNP(reference_file, input_vcf, snp_raw_vcf)
        SelectVariants.get_indel(reference_file, input_vcf, indel_raw_vcf)

        self.filter_bad_SNP(reference_file, snp_raw_vcf, snp_filtered_vcf, filter_name=snp_filter_name, QD=snp_QD,
                            FS=snp_FS, MQ=snp_MQ, #HaplotypeScore=snp_HaplotypeScore,
                            MappingQualityRankSum=snp_MappingQualityRankSum, ReadPosRankSum=snp_ReadPosRankSum)
        self.filter_bad_indel(reference_file, indel_raw_vcf, indel_filtered_vcf, filter_name=indel_filter_name,
                              QD=indel_QD, ReadPosRankSum=indel_ReadPosRankSum, FS=indel_FS)

        SelectVariants.remove_entries_with_filters(reference_file, snp_filtered_vcf, snp_good_vcf)
        SelectVariants.remove_entries_with_filters(reference_file, indel_filtered_vcf, indel_good_vcf)

        if combine_vcf:
            VCFRoutines.combine_same_samples_vcfs(unsorted_combined_filtered_vcf, vcf_list=[snp_filtered_vcf, indel_filtered_vcf],
                                                  order_vcf_files=False, sort=True, chunk_folder=None, chunk_prefix=None,
                                                  chunk_suffix=None, starting_chunk=None, chunk_number_list=None,
                                                  close_fd_after=False, extension_list=[".vcf", ])

            VCFRoutines.combine_same_samples_vcfs(unsorted_combined_good_vcf, vcf_list=[snp_good_vcf, indel_good_vcf],
                                                  order_vcf_files=False, sort=True, chunk_folder=None, chunk_prefix=None,
                                                  chunk_suffix=None, starting_chunk=None, chunk_number_list=None,
                                                  close_fd_after=False, extension_list=[".vcf", ])
            if sequence_dict_file:
                SortVcf.sort_vcf(unsorted_combined_filtered_vcf, combined_filtered_vcf, seq_dict=sequence_dict_file)
                SortVcf.sort_vcf(unsorted_combined_good_vcf, combined_good_vcf, seq_dict=sequence_dict_file)
            """
            #CombineVariants IS TOO SLOW!!!! It takes DAYS to merge VCFs
            CombineVariants.combine_from_same_source(reference_file, [snp_filtered_vcf, indel_filtered_vcf],
                                                     combined_filtered_vcf)

            CombineVariants.combine_from_same_source(reference_file, [snp_good_vcf, indel_good_vcf],
                                                     combined_good_vcf)
            """



__author__ = 'mahajrod'
import os
from RouToolPa.Routines import FileRoutines

#from Converters.Converters import gff22gff3, gff32gtf

def RepeatModeler_search(query_file, db_name, output_file="run.out",
                         num_of_threads=5, RepeatModeler_dir=""):
    print("\nRepeatModeler search...\n")
    repmod_dir = FileRoutines.check_path(RepeatModeler_dir)
    os.system(repmod_dir + "BuildDatabase -engine ncbi  -name %s %s" % (db_name, query_file))
    os.system(repmod_dir + "RepeatModeler -engine ncbi -pa %i -database %s > %s"
              % (num_of_threads, db_name, output_file))


def TRF_search(query_file, match=2, mismatch=7, delta=7, PM=80,
               PI=10, minscore=50, max_period=500, flanked=False, TRF_dir=""):

    print("\nTRF search...\n")
    #use: trf File Match Mismatch Delta PM PI Minscore MaxPeriod [options]
    #Where: (all weights, penalties, and scores are positive)
    # File = sequences input file
    # Match = matching weight
    # Mismatch = mismatching penalty
    # Delta = indel penalty
    # PM = match probability (whole number)
    # PI = indel probability (whole number)
    # Minscore = minimum alignment score to report
    # MaxPeriod = maximum period size to report
    # [options] = one or more of the following :
    # -m masked sequence file
    # -f flanking sequence
    # -d data file
    # -h suppress HTML output
    #Recomended options: trf yoursequence.txt 2 7 7 80 10 50 500 -f -d -m
    flanking = ""
    if flanked:
        flanking = "-f"

    trf_path = FileRoutines.check_path(TRF_dir)
    os.system(trf_path + "trf %s %i %i %i %i %i %i %i %s -d -m"
              % (query_file, match, mismatch, delta, PM, PI, minscore, max_period, flanking))


def extract_repbase(species, output_file="RepBase.fasta", RepeatMaskerUtils_dir=""):
    print("\nExtracting RepBase for %s\n" % species)
    repmaskutils_dir = FileRoutines.check_path(RepeatMaskerUtils_dir)
    os.system(repmaskutils_dir + "queryRepeatDatabase.pl -species %s > %s" % (species, output_file))


def RepeatMasker_search(query_file, species, custom_lib_path=None, RepeatMasker_dir="",
                        num_of_threads=5, search_type="-s"):

    #species: see list of possible species in repeatmasker.help coming with RepeatMasker
    #search type: "-s" (sensetive), "" (default), "-q" (fast), "-qq" (very fast)

    repmask_dir = FileRoutines.check_path(RepeatMasker_dir)
    custom_lib = ""
    if custom_lib_path:
        cuatom_lib = "-lib %s" % custom_lib_path

    #additional options:
    #-xm    creates an additional output file in cross_match format (for parsing)
    #-ace   creates an additional output file in ACeDB format
    #-gff   creates an additional Gene Feature Finding format
    #-excln The percentages displayed in the .tbl file are calculated using a
    #       total sequence length excluding runs of 25 Ns or more.
    print("\nRepeatMasker search...\n")
    os.system(repmask_dir + "RepeatMasker -excln -xm -ace -gff %s -pa %i -species %s %s %s"
              % (custom_lib, num_of_threads, species, search_type, query_file))


def rmout2gff3(rmoutfile, outfile, RepeatMaskerUtils_dir=""):
    repmaskutils_dir = FileRoutines.check_path(RepeatMaskerUtils_dir)
    os.system(repmaskutils_dir + "rmOutToGFF3.pl %s > %s" % (rmoutfile, outfile))


def windowmasker_search(windowmasker_dir):
    winmask_dir = FileRoutines.check_path(windowmasker_dir)
    #TODO: write this function
    pass

if __name__ == "__main__":
    reference_name = "LAN210_v0.3m"
    #reference_name = "LAN210_v0.4m"
    #reference_name = "LAN210_v0.5m"
    reference_file = reference_name + ".fasta"


    workdir = "/home/mahajrod/genetics/desaminases/data/%s/masking" % reference_name
    os.system("mkdir -p %s" % workdir)
    os.chdir(workdir)
    """
    os.system("ln -fs ../%s %s" % (reference_file, reference_file))
    os.system("mkdir -p repeatmodeler")
    os.chdir("repeatmodeler")
    RepeatModeler_search("../%s" % reference_file, reference_name,
                         RepeatModeler_dir="/home/mahajrod/Repositories/genetic/NGS_tools/RepeatModeler")
    os.chdir(workdir)
    os.system("mkdir -p custom_lib")
    #os.system("/bin/cp -rf ")
    extract_repbase("fungi", output_file="custom_lib/RepBase_fungi.fasta",
                    RepeatMaskerUtils_dir="/home/mahajrod/Repositories/genetic/NGS_tools/RepeatMasker/util")
    """
    #noncontinuos pipeline because of repeatmodeler - it creates diretory with data in name
    #put output of repeatmodeler (consensi.fa.classified) to custom_lib dir and combine it with repbase into custom_lib.fasta
    os.chdir(workdir)
    os.system("mkdir -p repeatmasker")
    os.chdir("repeatmasker")
    os.system("ln -fs ../%s %s" % (reference_file, reference_file))
    os.system("ln -fs ../custom_lib/custom_lib.fasta custom_lib.fasta")
    RepeatMasker_search(reference_file, "fungi", custom_lib_path="custom_lib.fasta")

    os.chdir(workdir)
    os.system("mkdir -p TRF")
    os.chdir("TRF")
    os.system("ln -fs ../repeatmasker/%s.masked %s_masked_repeatmasker.fasta" % (reference_file, reference_name))
    TRF_search("%s_masked_repeatmasker.fasta" % reference_name)



#!/usr/bin/env python
__author__ = 'Sergei F. Kliver'

import os
import re
from math import sqrt
from copy import deepcopy
from collections import OrderedDict, Iterable

import numpy as np

import matplotlib
matplotlib.use('Agg')
import matplotlib.pyplot as plt

from Bio import SeqIO
from Bio.SeqFeature import SeqFeature, FeatureLocation

from MACE.Parsers.Abstract import Record, Collection, Metadata, Header


class RecordSNPeff(Record):
    def __init__(self, pos, ref, alt_list, filter_list, effect, effect_impact, functional_class, codon_change,
                 amino_acid_change, amino_acid_len, gene_name, transcript_biotype, gene_coding, transcript_id, exon_rank,
                 genotype_number, errors, warnings, gene_name_alias_list=[], gene_function_list=[], gene_description=None,
                 info_dict=None, flags=None, biochemical_pathway=[]):

        Record.__init__(self, pos=pos, info_dict=info_dict, flags=flags)

        self.ref = ref
        self.alt_list = alt_list
        self.filter_list = filter_list
        self.effect = effect
        self.effect_impact = effect_impact
        self.functional_class = functional_class
        self.codon_change = codon_change
        self.amino_acid_change = amino_acid_change
        self.amino_acid_len = amino_acid_len
        self.gene_name = gene_name
        self.gene_name_alias_list = gene_name_alias_list
        self.gene_function_list = gene_function_list
        self.gene_description = gene_description
        self.transcript_biotype = transcript_biotype
        self.gene_coding = gene_coding
        self.transcript_id = transcript_id
        self.exon_rank = exon_rank
        self.genotype_number = genotype_number
        self.errors = errors
        self.warnings = warnings
        self.biochemical_pathway = biochemical_pathway

    def __str__(self):
        vcf_part = "%i\t%s\t%s\t%s" % (self.pos, self.ref, ",".join(self.alt_list) if self.alt_list else ".",
                                       ",".join(self.filter_list) if self.filter_list else ".")
        snpeff_part = ("\t" + "%s\t" * 17 + "%s") % (self.effect,
                                                     self.effect_impact,
                                                     self.functional_class,
                                                     self.codon_change,
                                                     self.amino_acid_change,
                                                     str(self.amino_acid_len) if self.amino_acid_len else ".",
                                                     self.gene_name,
                                                     self.transcript_biotype,
                                                     self.gene_coding,
                                                     self.transcript_id,
                                                     self.exon_rank,
                                                     self.genotype_number,
                                                     self.errors,
                                                     self.warnings,
                                                     ",".join(self.gene_name_alias_list) if self.gene_name_alias_list else ".",
                                                     ",".join(self.gene_function_list) if self.gene_function_list else ".",
                                                     ",".join(self.gene_description) if self.gene_description else ".",
                                                     ";".join(self.biochemical_pathway) if self.biochemical_pathway else ".")

        return vcf_part + snpeff_part


class HeaderSNPeff(list, Header):
    """
    HeaderVCF class
    """

    def __str__(self):
        """
        :return: tab-separatd representation of header
        """
        return "#" + "\t".join(self)


class CollectionSNPeff(Collection):

    def __init__(self, metadata=None, records_dict=None, header=None, input_file=None, from_file=False, filetype='tab'):
        # metadata should be Metadata class
        # header should be Header class
        # record_list should be list of Record class
        if from_file:
            self.read(input_file, filetype=filetype)
        else:
            self.records = OrderedDict() if records_dict is None else records_dict
            self.header = header
            self.metadata = metadata
        self.scaffold_list = self.scaffolds()
        self.scaffold_length = self.scaffold_len()
        self.number_of_scaffolds = len(self.scaffold_list)
        self.record_index = self.rec_index()

    def read(self, in_file, filetype='tab'):
        """
        Reads collection from vcf file
        :param in_file: path to file
        :param external_metadata: external(not from input file) metadata that could be used to parse records
        :return: None
        """
        self.metadata = None
        self.records = OrderedDict({})
        with open(in_file, "r") as fd:
            if filetype == 'tab':
                self.header = HeaderSNPeff(fd.readline()[1:].strip().split("\t"))
                gene_alias_presence = "Gene_Name_Aliases" in self.header
                gene_function_presence = "Gene_Functions" in self.header
                gene_description_presence = "Gene_Description" in self.header
                biochemical_pathway_presence = "Biochemical_Pathway" in self.header
                for line in fd:
                    scaffold, record = self.add_record_from_tab_line(line,
                                                                     gene_alias_presence=gene_alias_presence,
                                                                     gene_function_presence=gene_function_presence,
                                                                     gene_description_presence=gene_description_presence,
                                                                     biochemical_pathway_presence=biochemical_pathway_presence)
                    if scaffold not in self.records:
                        self.records[scaffold] = []
                    self.records[scaffold].append(record)

            elif filetype == 'vcf':
                raise ValueError("Parsing from vcf file annotated by SNPeff is not supported yet")

            else:
                raise ValueError("Wrong filetype for parsing")

        if not gene_alias_presence:
            self.header.append("Gene_Name_Aliases")
        if not gene_function_presence:
            self.header.append("Gene_Functions")
        if not gene_description_presence:
            self.header.append("Gene_Description")
        if not biochemical_pathway_presence:
            self.header.append("Biochemical_Pathway")

    def add_record_from_tab_line(self, line,
                                 gene_alias_presence,
                                 gene_function_presence,
                                 gene_description_presence,
                                 biochemical_pathway_presence):
        """
        Adds record to collection from line
        :param line: record line from tab file
        :return: None
        """
        line_list = line.strip().split("\t")
        pos = int(line_list[1])
        ref = line_list[2]
        alt_list = line_list[3].split(",")
        filter_list = line_list[4].split(",")
        effect = line_list[5]
        effect_impact = line_list[6]
        functional_class = line_list[7]
        codon_change = line_list[8]
        amino_acid_change = line_list[9]
        amino_acid_len = int(line_list[10]) if line_list[10] != "." else None
        gene_name = line_list[11]
        transcript_biotype = line_list[12]
        gene_coding = line_list[13]
        transcript_id = line_list[14]
        exon_rank = line_list[15]
        genotype_number = line_list[16]
        errors = line_list[17]
        warnings = line_list[18]
        gene_name_alias_list = line_list[self.header.index("Gene_Name_Aliases")].split(",") if gene_alias_presence else []
        gene_function_list = line_list[self.header.index("Gene_Functions")].split(",") if gene_function_presence else []
        gene_description = line_list[self.header.index("Gene_Description")].split(",") if gene_description_presence else []
        biochemical_pathway = line_list[self.header.index("Biochemical_Pathway")].split(",") if biochemical_pathway_presence else []

        return line_list[0], RecordSNPeff(pos, ref, alt_list, filter_list, effect, effect_impact,
                                          functional_class, codon_change,
                                          amino_acid_change, amino_acid_len, gene_name, transcript_biotype,
                                          gene_coding, transcript_id, exon_rank,
                                          genotype_number, errors, warnings,
                                          gene_name_alias_list=gene_name_alias_list,
                                          gene_function_list=gene_function_list,
                                          gene_description=gene_description,
                                          info_dict=None, flags=None,
                                          biochemical_pathway=biochemical_pathway)

    def add_gene_name_aliases(self, alias_dict):
        for scaffold in self.records:
            for record in self.records[scaffold]:
                if record.gene_name in alias_dict:
                    tmp_aliases = [alias_dict[record.gene_name]] if isinstance(alias_dict[record.gene_name], str) else alias_dict[record.gene_name]
                    new_aliases = []
                    for alias in tmp_aliases:
                        if alias != ".":
                            new_aliases.append(alias)
                    record.gene_name_alias_list = (record.gene_name_alias_list + new_aliases) if record.gene_name_alias_list else new_aliases

    def add_gene_functions(self, function_dict):
        for scaffold in self.records:
            for record in self.records[scaffold]:
                for gene_name in [record.gene_name] + record.gene_name_alias_list:
                    if gene_name in function_dict:
                        new_functions = [function_dict[gene_name]] if isinstance(function_dict[gene_name], str) else function_dict[gene_name]
                        record.gene_function_list = (record.gene_function_list + new_functions) if record.gene_function_list else new_functions
                        break

    def add_gene_description(self, description_dict):
        for scaffold in self.records:
            for record in self.records[scaffold]:
                for gene_name in [record.gene_name] + record.gene_name_alias_list:
                    if gene_name in description_dict:
                        new_descriptions = [description_dict[gene_name]] if isinstance(description_dict[gene_name], str) else description_dict[gene_name]
                        record.gene_description = (record.gene_description + new_descriptions) if record.gene_description else new_descriptions
                        break

    def add_biochemical_pathway(self, biochemical_pathway_dict):
        for scaffold in self.records:
            for record in self.records[scaffold]:
                for gene_name in [record.gene_name] + record.gene_name_alias_list:
                    if gene_name in biochemical_pathway_dict:
                        new_pathway = [biochemical_pathway_dict[gene_name]] if isinstance(biochemical_pathway_dict[gene_name], str) else biochemical_pathway_dict[gene_name]
                        record.biochemical_pathway = (record.biochemical_pathway + new_pathway) if record.biochemical_pathway else new_pathway
                        break



#!/usr/bin/env python
__author__ = 'Sergei F. Kliver'

import os
from RouToolPa.Tools.Abstract import Tool


class GapFiller(Tool):
    def __init__(self, path="", max_threads=4):
        Tool.__init__(self, "SSPACE_Standard_v3.0.pl", path=path, max_threads=max_threads)

    def close_gaps(self, library_file, scaffold_fasta, output_basename, min_overlap_len=None,
                   min_read_number_for_base_calling=None, read_with_extention_fraction_for_closing=None,
                   max_gap_len_difference_to_stop=None, num_reads_to_trim=None, iteration_number=None,
                   min_contig_overlap=None, max_gaps_for_bowtie=None, skip_read_processing=None):

        options = " -T %i" % self.threads
        options += " -l %s" % library_file
        options += " -s %s" % scaffold_fasta

        options += " -m %i" % min_overlap_len if min_overlap_len else ""
        options += " -o %i" % min_read_number_for_base_calling if min_read_number_for_base_calling else ""
        options += " -r %f" % read_with_extention_fraction_for_closing if read_with_extention_fraction_for_closing else ""
        options += " -d %i" % max_gap_len_difference_to_stop if max_gap_len_difference_to_stop else ""
        options += " -t %i" % num_reads_to_trim if num_reads_to_trim else ""
        options += " -n %i" % min_contig_overlap if min_contig_overlap else ""
        options += " -i %i" % iteration_number if iteration_number else ""
        options += " -g %i" % max_gaps_for_bowtie if max_gaps_for_bowtie else ""
        options += " -S" if skip_read_processing else ""
        options += " -b %s" % output_basename if output_basename else ""

        self.execute(options)

if __name__ == "__main__":
    pass





#!/usr/bin/env python
"""
Last output parser Module based on pandas
"""
__author__ = 'Sergei F. Kliver'
import datetime
from copy import deepcopy
from collections import OrderedDict

import numpy as np
import pandas as pd

import RouToolPa.Formats.AlignmentFormats as AlignmentFormats


class CollectionLast:

    def __init__(self, in_file=None, records=None, format="tab", parsing_mode="all",
                 target_black_list=(), target_white_list=(),
                 query_black_list=(), query_white_list=(),
                 target_syn_dict=None, query_syn_dict=None,
                 min_target_hit_len=None, min_query_hit_len=None,
                 min_target_len=None, min_query_len=None):

        self.formats = ["tab", "tab_mismap"]
        self.TAB_COLS = AlignmentFormats.LAST_TAB_COLS
        self.parsing_parameters = {"tab": {
                                           "coordinates_only": {
                                                   "col_names": ["target_id", "target_start", "target_hit_len",
                                                                 "target_strand", "target_len", "query_id",
                                                                 "query_start", "query_hit_len", "query_strand",
                                                                 "query_len"],
                                                   "cols":      [1, 2, 3, 4, 5, 6, 7, 8, 9, 10],
                                                   "index_cols": None,
                                                   "converters": {
                                                                  "target_id":      str,
                                                                  "target_start":   np.int64,
                                                                  "target_hit_len": np.int64,
                                                                  "target_strand":  str,
                                                                  "target_len":     np.int64,
                                                                  "query_id":       str,
                                                                  "query_start":    np.int64,
                                                                  "query_hit_len":  np.int64,
                                                                  "query_strand":   str,
                                                                  "query_len":      np.int64,
                                                                  },
                                                   "col_name_indexes": {
                                                                        "target_id":   0,
                                                                        "target_start": 1,
                                                                        "target_hit_len": 2,
                                                                        "target_strand": 3,
                                                                        "target_len": 4,
                                                                        "query_id": 5,
                                                                        "query_start": 6,
                                                                        "query_hit_len": 7,
                                                                        "query_strand": 8,
                                                                        "query_len": 9,
                                                                        },
                                                                 },

                                           "all": {
                                                   "col_names": ["score", "target_id", "target_start", "target_hit_len",
                                                                 "target_strand", "target_len", "query_id",
                                                                 "query_start", "query_hit_len", "query_strand",
                                                                 "query_len", "alignment", "EG", "E"],
                                                   "cols":      [0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13],
                                                   "index_cols": None,
                                                   "converters": {
                                                                  "score":          np.int64,
                                                                  "target_id":      str,
                                                                  "target_start":   np.int64,
                                                                  "target_hit_len": np.int64,
                                                                  "target_strand":  str,
                                                                  "target_len":     np.int64,
                                                                  "query_id":       str,
                                                                  "query_start":    np.int64,
                                                                  "query_hit_len":  np.int64,
                                                                  "query_strand":   str,
                                                                  "query_len":      np.int64,
                                                                  "alignment":      str,
                                                                  "EG":             str,
                                                                  "E":              str,
                                                                  },
                                                   "col_name_indexes": {
                                                                        "score": 0,
                                                                        "target_id":   1,
                                                                        "target_start": 2,
                                                                        "target_hit_len": 3,
                                                                        "target_strand": 4,
                                                                        "target_len": 5,
                                                                        "query_id": 6,
                                                                        "query_start": 7,
                                                                        "query_hit_len": 8,
                                                                        "query_strand": 9,
                                                                        "query_len": 10,
                                                                        "alignment": 11,
                                                                        "EG": 12,
                                                                        "E": 13,
                                                                        },
                                                   },
                                           "complete": {
                                                        "col_names": ["score", "target_id", "target_start", "target_hit_len",
                                                                      "target_strand", "target_len", "query_id",
                                                                      "query_start", "query_hit_len", "query_strand",
                                                                      "query_len", "alignment", "EG", "E"],
                                                        "cols":      [0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13],
                                                        "index_cols": None,
                                                        "converters": {
                                                                       "score":          np.int64,
                                                                       "target_id":      str,
                                                                       "target_start":   np.int64,
                                                                       "target_hit_len": np.int64,
                                                                       "target_strand":  str,
                                                                       "target_len":     np.int64,
                                                                       "query_id":       str,
                                                                       "query_start":    np.int64,
                                                                       "query_hit_len":  np.int64,
                                                                       "query_strand":   str,
                                                                       "query_len":      np.int64,
                                                                       "alignment":      str,
                                                                       "EG":             str,
                                                                       "E":              str,
                                                                       },
                                                        "col_name_indexes": {
                                                                             "score": 0,
                                                                             "target_id":   1,
                                                                             "target_start": 2,
                                                                             "target_hit_len": 3,
                                                                             "target_strand": 4,
                                                                             "target_len": 5,
                                                                             "query_id": 6,
                                                                             "query_start": 7,
                                                                             "query_hit_len": 8,
                                                                             "query_strand": 9,
                                                                             "query_len": 10,
                                                                             "alignment": 11,
                                                                             "EG": 12,
                                                                             "E": 13,
                                                                             },
                                                   },
                                           },

                                "tab_mismap": {
                                                "coordinates_only": {
                                                    "col_names": ["target_id", "target_start", "target_hit_len",
                                                                  "target_strand", "target_len", "query_id",
                                                                  "query_start", "query_hit_len", "query_strand",
                                                                  "query_len"],
                                                    "cols": [1, 2, 3, 4, 5, 6, 7, 8, 9, 10],
                                                    "index_cols": None,
                                                    "converters": {
                                                        "target_id": str,
                                                        "target_start": np.int64,
                                                        "target_hit_len": np.int64,
                                                        "target_strand": str,
                                                        "target_len": np.int64,
                                                        "query_id": str,
                                                        "query_start": np.int64,
                                                        "query_hit_len": np.int64,
                                                        "query_strand": str,
                                                        "query_len": np.int64,
                                                    },
                                                    "col_name_indexes": {
                                                        "target_id": 0,
                                                        "target_start": 1,
                                                        "target_hit_len": 2,
                                                        "target_strand": 3,
                                                        "target_len": 4,
                                                        "query_id": 5,
                                                        "query_start": 6,
                                                        "query_hit_len": 7,
                                                        "query_strand": 8,
                                                        "query_len": 9,
                                                    },
                                                },

                                                "all": {
                                                    "col_names": ["score", "target_id", "target_start", "target_hit_len",
                                                                  "target_strand", "target_len", "query_id",
                                                                  "query_start", "query_hit_len", "query_strand",
                                                                  "query_len", "alignment", "mismap"],
                                                    "cols": [0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12],
                                                    "index_cols": None,
                                                    "converters": {
                                                        "score": np.int64,
                                                        "target_id": str,
                                                        "target_start": np.int64,
                                                        "target_hit_len": np.int64,
                                                        "target_strand": str,
                                                        "target_len": np.int64,
                                                        "query_id": str,
                                                        "query_start": np.int64,
                                                        "query_hit_len": np.int64,
                                                        "query_strand": str,
                                                        "query_len": np.int64,
                                                        "alignment": str,
                                                        "mismap": str,
                                                    },
                                                    "col_name_indexes": {
                                                        "score": 0,
                                                        "target_id": 1,
                                                        "target_start": 2,
                                                        "target_hit_len": 3,
                                                        "target_strand": 4,
                                                        "target_len": 5,
                                                        "query_id": 6,
                                                        "query_start": 7,
                                                        "query_hit_len": 8,
                                                        "query_strand": 9,
                                                        "query_len": 10,
                                                        "alignment": 11,
                                                        "mismap": 12,
                                                    },
                                                },
                                                "complete": {
                                                    "col_names": ["score", "target_id", "target_start", "target_hit_len",
                                                                  "target_strand", "target_len", "query_id",
                                                                  "query_start", "query_hit_len", "query_strand",
                                                                  "query_len", "alignment", "mismap"],
                                                    "cols": [0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12],
                                                    "index_cols": None,
                                                    "converters": {
                                                        "score": np.int64,
                                                        "target_id": str,
                                                        "target_start": np.int64,
                                                        "target_hit_len": np.int64,
                                                        "target_strand": str,
                                                        "target_len": np.int64,
                                                        "query_id": str,
                                                        "query_start": np.int64,
                                                        "query_hit_len": np.int64,
                                                        "query_strand": str,
                                                        "query_len": np.int64,
                                                        "alignment": str,
                                                        "mismap": str
                                                    },
                                                    "col_name_indexes": {
                                                        "score": 0,
                                                        "target_id": 1,
                                                        "target_start": 2,
                                                        "target_hit_len": 3,
                                                        "target_strand": 4,
                                                        "target_len": 5,
                                                        "query_id": 6,
                                                        "query_start": 7,
                                                        "query_hit_len": 8,
                                                        "query_strand": 9,
                                                        "query_len": 10,
                                                        "alignment": 11,
                                                        "mismap": 12,
                                                    },
                                                },
                                },
                                   }
        self.parsing_mode = parsing_mode
        self.alignment_parsing_modes = ["all", "complete"]
        #self.attributes_parsing_modes = ["complete", "coord_and_attr"]
        self.format = format
        self.target_black_list = target_black_list
        self.target_white_list = target_white_list
        self.target_syn_dict = target_syn_dict

        self.query_black_list = query_black_list
        self.query_white_list = query_white_list
        self.query_syn_dict = query_syn_dict
        # attributes type conversion parameters

        self.converters = OrderedDict()
        self.pandas_int_type_correspondence = OrderedDict({
                                                           "Int8":  np.float16,
                                                           "Int16": np.float16,
                                                           "Int32": np.float32,
                                                           "Int64": np.float64,
                                                           })

        # init aliases
        self.col_names = self.parsing_parameters[self.format][self.parsing_mode]["col_names"]
        self.index_cols = self.parsing_parameters[self.format][self.parsing_mode]["index_cols"]
        self.target_id_index = self.parsing_parameters[self.format][self.parsing_mode]["col_name_indexes"]["target_id"]
        self.target_start_index = self.parsing_parameters[self.format][self.parsing_mode]["col_name_indexes"]["target_start"]
        self.target_hit_len_index = self.parsing_parameters[self.format][self.parsing_mode]["col_name_indexes"]["target_hit_len"]
        self.target_strand_index = self.parsing_parameters[self.format][self.parsing_mode]["col_name_indexes"]["target_strand"]

        self.query_id_index = self.parsing_parameters[self.format][self.parsing_mode]["col_name_indexes"]["query_id"]
        self.query_start_index = self.parsing_parameters[self.format][self.parsing_mode]["col_name_indexes"]["query_start"]
        self.query_hit_len_index = self.parsing_parameters[self.format][self.parsing_mode]["col_name_indexes"]["query_hit_len"]
        self.query_strand_index = self.parsing_parameters[self.format][self.parsing_mode]["col_name_indexes"]["query_strand"]

        #
        self.target_scaffold_list = None
        self.query_scaffold_list = None
        self.target_scaffold_lengths = None
        self.query_scaffold_lengths = None

        if in_file:
            self.read(in_file,
                      format=format,
                      parsing_mode=parsing_mode,
                      target_black_list=target_black_list,
                      target_white_list=target_white_list,
                      query_black_list=query_black_list,
                      query_white_list=query_white_list,
                      min_target_hit_len=min_target_hit_len,
                      min_query_hit_len=min_query_hit_len,
                      min_target_len=min_target_len,
                      min_query_len=min_query_len)

        else:
            self.records = records

    def read(self, in_file,
             format="tab", parsing_mode="only_coordinates",
             target_black_list=(), target_white_list=(),
             query_black_list=(), query_white_list=(),
             min_target_hit_len=None, min_query_hit_len=None,
             min_target_len=None, min_query_len=None):
        if format not in self.parsing_parameters:
            raise ValueError("ERROR!!! This format(%s) was not implemented yet for parsing!" % parsing_mode)
        elif parsing_mode not in self.parsing_parameters[format]:
            raise ValueError("ERROR!!! This format(%s) was not implemented yet for parsing in this mode(%s)!" % (format, parsing_mode))

        print("%s\tReading input..." % str(datetime.datetime.now()))
        self.records = pd.read_csv(in_file, sep='\t', header=None, na_values=".",
                                   comment="#",
                                   usecols=self.parsing_parameters[format][parsing_mode]["cols"],
                                   converters=self.parsing_parameters[format][parsing_mode]["converters"],
                                   names=self.parsing_parameters[format][parsing_mode]["col_names"],
                                   index_col=self.parsing_parameters[format][parsing_mode]["index_cols"])

        print("%s\tReading input finished..." % str(datetime.datetime.now()))
        print("%s\tFiltering..." % str(datetime.datetime.now()))
        if target_white_list or target_black_list:
            target_scaffolds_to_keep = self.get_filtered_entry_list(self.records["target_id"].tolist(),
                                                                    entry_black_list=target_black_list,
                                                                    entry_white_list=target_white_list)
            #print target_scaffolds_to_keep
            self.records = self.records[self.records["target_id"].isin(target_scaffolds_to_keep)]

        if query_white_list or query_black_list:
            query_scaffolds_to_keep = self.get_filtered_entry_list(self.records["query_id"].tolist(),
                                                                   entry_black_list=query_black_list,
                                                                   entry_white_list=query_white_list)
            #print query_scaffolds_to_keep
            self.records = self.records[self.records["query_id"].isin(query_scaffolds_to_keep)]
        if self.target_syn_dict:
            self.records["target_id"].replace(self.target_syn_dict, inplace=True)
        if self.query_syn_dict:
            self.records["query_id"].replace(self.query_syn_dict, inplace=True)
        # retain only automatically generated index by row number
        #self.records.index = pd.MultiIndex.from_arrays([self.records.index, np.arange(0, len(self.records))],
        #                                               names=("scaffold", "row"))
        print("%s\tFiltering finished..." % str(datetime.datetime.now()))

        if min_target_len and min_query_len:
            self.records = self.records[(self.records["target_len"] >= min_target_len) & (self.records["query_len"] >= min_query_len)]
        elif min_target_len:
            self.records = self.records[self.records["target_len"] >= min_target_len]
        elif min_query_len:
            self.records = self.records[self.records["query_len"] >= min_query_len]
            
        if min_target_hit_len and min_query_hit_len:
            self.records = self.records[(self.records["target_hit_len"] >= min_target_hit_len) & (self.records["query_hit_len"] >= min_query_hit_len)]
        elif min_target_hit_len:
            self.records = self.records[self.records["target_hit_len"] >= min_target_hit_len]
        elif min_query_hit_len:
            self.records = self.records[self.records["query_hit_len"] >= min_query_hit_len]

        self.target_scaffold_lengths = self.records[["target_id", "target_len"]].drop_duplicates()
        self.target_scaffold_lengths.columns = ["id", "length"]
        self.target_scaffold_lengths.set_index("id", inplace=True)
        self.query_scaffold_lengths = self.records[["query_id", "query_len"]].drop_duplicates()
        self.query_scaffold_lengths.columns = ["id", "length"]
        self.query_scaffold_lengths.set_index("id", inplace=True)
        self.target_scaffold_list = self.target_scaffold_lengths.index.tolist()
        self.query_scaffold_list = self.query_scaffold_lengths.index.tolist()

        retained_columns = deepcopy(self.parsing_parameters[self.format][self.parsing_mode]["col_names"])
        for entry in "target_len", "query_len":
            retained_columns.remove(entry)

        self.records = self.records[retained_columns]

        if (self.format == "tab") and (parsing_mode == "complete"):
            self.records["EG2"] = map(lambda s: np.float32(s.split("=")[1]), list(self.records["EG2"]))
            self.records["E"] = map(lambda s: np.float32(s.split("=")[1]), list(self.records["E"]))
        elif (self.format == "tab_mismap") and (parsing_mode == "complete"):
            self.records["mismap"] = map(lambda s: np.float32(s.split("=")[1]), list(self.records["mismap"]))

    def sort(self, inplace=False,
             sorting_order=("target_id", "query_id", "target_start", "target_hit_len", "query_start", "query_hit_len")):
        return self.records.sort_values(by=sorting_order, inplace=inplace)

    @staticmethod
    def get_filtered_entry_list(entry_list,
                                entry_black_list=[],
                                sort_entries=False,
                                entry_ordered_list=None,
                                entry_white_list=[]):
        white_set = set(entry_white_list)
        black_set = set(entry_black_list)
        entry_set = set(entry_list)

        if white_set:
            entry_set = entry_set & white_set
        if black_set:
            entry_set = entry_set - black_set

        filtered_entry_list = list(entry_set)
        if sort_entries:
            filtered_entry_list.sort()

        final_entry_list = []

        if entry_ordered_list:
            for entry in entry_ordered_list:
                if entry in filtered_entry_list:
                    final_entry_list.append(entry)
                    filtered_entry_list.remove(entry)
                else:
                    print("WARNING!!!Entry(%s) from order list is absent in list of entries!" % entry)
            return final_entry_list + filtered_entry_list
        else:
            return filtered_entry_list

    @staticmethod
    def last_alignment_to_(last_aln_str):

        splited_string = map(lambda s: s.split[":"], last_aln_str.split(","))

    def get_coordinates_from_(self):
        pass

    def write(self, output, separator="\t", header=False):
        self.records.to_csv(output,
                            sep=separator,
                            header=header,
                            index=False)

    def merge_adjucent_blocks(self, max_dist_to_collapse=None):
        """
        ONLY COORDINATES ARE MERGED. OTHER FIELDS HAVE NO SENSE AFTER THIS PROCEDUREAND ARE RETAINED ONLY FOR PARSING COMPATIBILITY!
        :return:
        """
        row_iterator = self.records.itertuples(index=False)
        merged_row_list = []

        for row in row_iterator:
            curr_row = list(row)
            for index in self.target_id_index, self.target_strand_index, self.query_id_index, self.query_strand_index:

                if prev_row[index] != curr_row[index]:
                    merged_row_list.append(prev_row)
                    prev_row = curr_row
                    break
            else:
                if max_dist_to_collapse:
                    if (curr_row[self.target_start_index] - prev_row[self.target_start_index] -
                                prev_row[self.target_hit_len_index] + 1 <= max_dist_to_collapse) and \
                            (curr_row[self.query_start_index] - prev_row[self.query_start_index] -
                                 prev_row[self.query_hit_len_index] <= max_dist_to_collapse):
                        prev_row[self.target_hit_len_index] += curr_row[self.target_hit_len_index]
                        prev_row[self.query_hit_len_index] += curr_row[self.query_hit_len_index]
                    else:
                        merged_row_list.append(prev_row)
                        prev_row = curr_row
                else:
                    prev_row[self.target_hit_len_index] += curr_row[self.target_hit_len_index]
                    prev_row[self.query_hit_len_index] += curr_row[self.query_hit_len_index]

        merged_row_list.append(prev_row)

    def extract_coordinates(self):

        return self.records[["target_id", "target_start", "target_hit_len", "target_strand",
                             "query_id", "query_start", "query_hit_len", "query_strand"]]

    def extract_coordinates_to_file(self, output, separator="\t", header=False):
        self.records.to_csv(output,
                            columns=["target_id", "target_start", "target_hit_len", "target_strand",
                                     "query_id", "query_start", "query_hit_len", "query_strand"],
                            sep=separator,
                            header=header,
                            index=False)

    def extract_and_convert_coordinates(self, output=None, separator="\t", header=False):
        coords = deepcopy(self.records[["target_id", "target_start", "target_hit_len", "target_strand",
                                        "query_id", "query_start", "query_hit_len", "query_strand"]])
        coords.columns = ["target_id", "target_start", "target_end", "target_strand",
                        "query_id", "query_start", "query_end", "query_strand"]
        coords["target_end"] = coords["target_start"] + coords["target_end"]
        coords["query_end"] = coords["query_start"] + coords["query_end"]
        if output:
            coords.to_csv(output, sep=separator, header=header,
                          index=False)

        return coords

    def collapse_coordinates(self, output=None, max_dist_to_collapse=1000, separator="\t", header=False):

        coords = self.extract_and_convert_coordinates()
        row_iterator = coords.itertuples(index=False)

        prev_row = list(row_iterator.next())
        collapsed_row_list = []

        for row in row_iterator:
            curr_row = list(row)
            for index in 0, 3, 4, 7:
                if prev_row[index] != curr_row[index]:
                    collapsed_row_list.append(prev_row)
                    prev_row = curr_row
                    break
            else:
                if max_dist_to_collapse:
                    if (curr_row[1] - prev_row[2] <= max_dist_to_collapse) and (curr_row[5] - prev_row[6] <= max_dist_to_collapse):
                        prev_row[2] = curr_row[2]
                        prev_row[6] = curr_row[6]
                    else:
                        collapsed_row_list.append(prev_row)
                        prev_row = curr_row
                else:
                    prev_row[2] = curr_row[2]
                    prev_row[6] = curr_row[6]

        collapsed_row_list.append(prev_row)

        collapsed_row_list = pd.DataFrame.from_records(collapsed_row_list, columns=coords.columns)

        if output:
            collapsed_row_list.to_csv(output, sep=separator, header=header, index=False)

        return collapsed_row_list

    def rename_target_ids(self, syn_dict):
        self.records[["target_id"]].replace(syn_dict, inplace=True)

    def rename_query_ids(self, syn_dict):
        self.records[["query_id"]].replace(syn_dict, inplace=True)

    @staticmethod
    def parse_alignment_string(alignment_string):

        str_list = map(lambda s: s.split(":"), alignment_string.split(","))

        target_list = []
        query_list = []

        for entry in str_list:
            target_list.append(int(entry[0]))
            query_list.append(int(entry[0]) if len(entry) == 1 else int(entry[1]))
        
        return np.array(target_list), np.array(query_list)




#!/usr/bin/env python
__author__ = 'Sergei F. Kliver'

import os
from RouToolPa.Tools.Abstract import Tool


class Supernova(Tool):
    def __init__(self, path="", max_threads=4):
        Tool.__init__(self, "supernova", path=path, max_threads=max_threads)


#!/usr/bin/env python
import os
from RouToolPa.Tools.Abstract import Tool


class RepeatScout(Tool):
    def __init__(self, path="", max_threads=1):
        Tool.__init__(self, "RepeatScout", path=path, max_threads=max_threads)

    def predict_repeats(self, sequence, output, freq, l):
        """
        Usage:
        RepeatScout -sequence <seq> -output <out> -freq <freq> -l <l> [opts]
             -L # size of region to extend left or right (10000)
             -match # reward for a match (+1)
             -mismatch # penalty for a mismatch (-1)
             -gap  # penalty for a gap (-5)
             -maxgap # maximum number of gaps allowed (5)
             -maxoccurrences # cap on the number of sequences to align (10,000)
             -maxrepeats # stop work after reporting this number of repeats (10000)
             -cappenalty # cap on penalty for exiting alignment of a sequence (-20)
             -tandemdist # of bases that must intervene between two l-mers for both to be counted (500)
             -minthresh # stop if fewer than this number of l-mers are found in the seeding phase (3)
             -minimprovement # amount that a the alignment needs to improve each step to be considered progress (3)
             -stopafter # stop the alignment after this number of no-progress columns (100)
             -goodlength # minimum required length for a sequence to be reported (50)
             -maxentropy # entropy (complexity) threshold for an l-mer to be considered (-.7)
             -v[v[v[v]]] How verbose do you want it to be?  -vvvv is super-verbose.
        """

        options = " -sequence %s" % sequence
        options += " -output %s" % output
        options += " -freq %s" % str(freq)
        options += " -l %s" % str(l)

        self.execute(options)